@@ -27,65 +27,25 @@ B<App::Raps2> is the backend for B<raps2>, a simple commandline password safe.
 
 =cut
 
-
 use strict;
 use warnings;
 use autodie;
 use 5.010;
 
-use base 'Exporter';
-
 use App::Raps2::Password;
 use App::Raps2::UI;
-use Carp q(confess);
+use Carp qw(confess);
+use File::BaseDir qw(config_home data_home);
 use File::Path qw(make_path);
-use File::Slurp qw(slurp write_file);
+use File::Slurp qw(read_dir slurp write_file);
 
-our @EXPORT_OK = ();
-our $VERSION = '0.1';
+our $VERSION = '0.2';
 
 =head1 METHODS
 
-=head2 create_salt()
+=over
 
-Returns a 16-character random salt for App::Raps2::Password(3pm).
-
-=cut
-
-sub create_salt {
-	my $salt = q{};
-
-	for (1 .. 16) {
-		$salt .= chr(0x21 + int(rand(90)));
-	}
-
-	return $salt;
-}
-
-=head2 file_to_hash($file)
-
-Reads $file (lines with key/value separated by whitespace) and returns a hash
-with its key/value pairs.
-
-=cut
-
-sub file_to_hash {
-	my ($file) = @_;
-	my %ret;
-
-	for my $line (slurp($file)) {
-		my ($key, $value) = split(qr{\s+}, $line);
-
-		if (not ($key and $value)) {
-			next;
-		}
-
-		$ret{$key} = $value;
-	}
-	return %ret;
-}
-
-=head2 new(%conf)
+=item $raps2 = App::Raps2->new(I<%conf>)
 
 Returns a new B<App::Raps2> object.
 
@@ -105,18 +65,57 @@ sub new {
 	my ($obj, %conf) = @_;
 	my $ref = {};
 
-	$ref->{'xdg_conf'} = $ENV{'XDG_CONFIG_HOME'} // "$ENV{HOME}/.config/raps2";
-	$ref->{'xdg_data'} = $ENV{'XDG_DATA_HOME'} //
-		"$ENV{HOME}/.local/share/raps2";
+	$ref->{xdg_conf} = config_home('raps2');
+	$ref->{xdg_data} = data_home('raps2');
 
-	$ref->{'ui'} = App::Raps2::UI->new();
+	$ref->{ui} = App::Raps2::UI->new();
 
-	$ref->{'default'} = \%conf;
+	$ref->{default} = \%conf;
 
 	return bless($ref, $obj);
 }
 
-=head2 ->sanity_check()
+=item $raps2->create_salt()
+
+Returns a 16-character random salt for App::Raps2::Password(3pm).
+
+=cut
+
+sub create_salt {
+	my ($self) = @_;
+	my $salt = q{};
+
+	for (1 .. 16) {
+		$salt .= chr(0x21 + int(rand(90)));
+	}
+
+	return $salt;
+}
+
+=item $raps2->file_to_hash(I<$file>)
+
+Reads $file (lines with key/value separated by whitespace) and returns a hash
+with its key/value pairs.
+
+=cut
+
+sub file_to_hash {
+	my ($self, $file) = @_;
+	my %ret;
+
+	for my $line (slurp($file)) {
+		my ($key, $value) = split(qr{\s+}, $line);
+
+		if (not ($key and $value)) {
+			next;
+		}
+
+		$ret{$key} = $value;
+	}
+	return %ret;
+}
+
+=item $raps2->sanity_check()
 
 Create working directories (~/.config/raps2 and ~/.local/share/raps2, or the
 respective XDG environment variable contents), if they don't exist yet.
@@ -128,17 +127,17 @@ Calls B<create_config> if no raps2 config was found.
 sub sanity_check {
 	my ($self) = @_;
 
-	make_path($self->{'xdg_conf'});
-	make_path($self->{'xdg_data'});
+	make_path($self->{xdg_conf});
+	make_path($self->{xdg_data});
 
-	if (not -e $self->{'xdg_conf'} . '/password') {
+	if (not -e $self->{xdg_conf} . '/password') {
 		$self->create_config();
 	}
 
 	return;
 }
 
-=head2 ->get_master_password()
+=item $raps2->get_master_password()
 
 Asks the user for the master passphrase.
 
@@ -148,16 +147,16 @@ sub get_master_password {
 	my ($self) = @_;
 	my $pass = $self->ui()->read_pw('Master Password', 0);
 
-	$self->{'pass'} = App::Raps2::Password->new(
-		cost => $self->{'default'}->{'cost'},
-		salt => $self->{'master_salt'},
+	$self->{pass} = App::Raps2::Password->new(
+		cost => $self->{default}->{cost},
+		salt => $self->{master_salt},
 		passphrase => $pass,
 	);
 
-	$self->{'pass'}->verify($self->{'master_hash'});
+	$self->{pass}->verify($self->{master_hash});
 }
 
-=head2 ->create_config()
+=item $raps2->create_config()
 
 Creates a default config and asks the user to set a master password.
 
@@ -166,25 +165,25 @@ Creates a default config and asks the user to set a master password.
 sub create_config {
 	my ($self) = @_;
 	my $cost = 12;
-	my $salt = create_salt();
+	my $salt = $self->create_salt();
 	my $pass = $self->ui()->read_pw('Master Password', 1);
 
-	$self->{'pass'} = App::Raps2::Password->new(
+	$self->{pass} = App::Raps2::Password->new(
 		cost => $cost,
 		salt => $salt,
 		passphrase => $pass,
 	);
-	my $hash = $self->{'pass'}->crypt();
+	my $hash = $self->{pass}->crypt();
 
 	write_file(
-		$self->{'xdg_conf'} . '/password',
+		$self->{xdg_conf} . '/password',
 		"cost ${cost}\n",
 		"salt ${salt}\n",
 		"hash ${hash}\n",
 	);
 }
 
-=head2 ->load_config()
+=item $raps2->load_config()
 
 Load config
 
@@ -192,13 +191,13 @@ Load config
 
 sub load_config {
 	my ($self) = @_;
-	my %cfg = file_to_hash($self->{'xdg_conf'} . '/password');
-	$self->{'master_hash'} = $cfg{'hash'};
-	$self->{'master_salt'} = $cfg{'salt'};
-	$self->{'default'}->{'cost'} //= $cfg{'cost'};
+	my %cfg = $self->file_to_hash($self->{xdg_conf} . '/password');
+	$self->{master_hash} = $cfg{hash};
+	$self->{master_salt} = $cfg{salt};
+	$self->{default}->{cost} //= $cfg{cost};
 }
 
-=head2 ->ui()
+=item $raps2->ui()
 
 Returns the App::Raps2::UI(3pm) object.
 
@@ -206,10 +205,10 @@ Returns the App::Raps2::UI(3pm) object.
 
 sub ui {
 	my ($self) = @_;
-	return $self->{'ui'};
+	return $self->{ui};
 }
 
-=head2 ->cmd_add($name)
+=item $raps2->cmd_add(I<$name>)
 
 Adds a new password file called $name.
 
@@ -217,7 +216,7 @@ Adds a new password file called $name.
 
 sub cmd_add {
 	my ($self, $name) = @_;
-	my $pwfile = $self->{'xdg_data'} . "/${name}";
+	my $pwfile = $self->{xdg_data} . "/${name}";
 
 	if (-e $pwfile) {
 		confess('Password file already exists');
@@ -225,17 +224,17 @@ sub cmd_add {
 
 	$self->get_master_password();
 
-	my $salt = create_salt();
+	my $salt = $self->create_salt();
 	my $url = $self->ui()->read_line('URL');
 	my $login = $self->ui()->read_line('Login');
 	my $pass = $self->ui()->read_pw('Password', 1);
 	my $extra = $self->ui()->read_multiline('Additional content');
 
-	$self->{'pass'}->salt($salt);
-	my $pass_hash = $self->{'pass'}->encrypt($pass);
+	$self->{pass}->salt($salt);
+	my $pass_hash = $self->{pass}->encrypt($pass);
 	my $extra_hash = (
 		$extra ?
-		$self->{'pass'}->encrypt($extra) :
+		$self->{pass}->encrypt($extra) :
 		q{}
 	);
 
@@ -250,7 +249,7 @@ sub cmd_add {
 	);
 }
 
-=head2 ->cmd_dump($name)
+=item $raps2->cmd_dump(I<$name>)
 
 Dumps the content of $name.
 
@@ -258,29 +257,29 @@ Dumps the content of $name.
 
 sub cmd_dump {
 	my ($self, $name) = @_;
-	my $pwfile = $self->{'xdg_data'} . "/${name}";
+	my $pwfile = $self->{xdg_data} . "/${name}";
 
 	if (not -e $pwfile) {
 		confess('Password file does not exist');
 	}
 
-	my %key = file_to_hash($pwfile);
+	my %key = $self->file_to_hash($pwfile);
 
 	$self->get_master_password();
 
-	$self->{'pass'}->salt($key{'salt'});
+	$self->{pass}->salt($key{salt});
 
 	$self->ui()->output(
-		['URL', $key{'url'}],
-		['Login', $key{'login'}],
-		['Password', $self->{'pass'}->decrypt($key{'hash'})],
+		['URL', $key{url}],
+		['Login', $key{login}],
+		['Password', $self->{pass}->decrypt($key{hash})],
 	);
-	if ($key{'extra'}) {
-		print $self->{'pass'}->decrypt($key{'extra'});
+	if ($key{extra}) {
+		print $self->{pass}->decrypt($key{extra});
 	}
 }
 
-=head2 ->cmd_get($name)
+=item $raps2->cmd_get(I<$name>)
 
 Puts the password saved in $name into the X clipboard.
 
@@ -288,26 +287,26 @@ Puts the password saved in $name into the X clipboard.
 
 sub cmd_get {
 	my ($self, $name) = @_;
-	my $pwfile = $self->{'xdg_data'} . "/${name}";
+	my $pwfile = $self->{xdg_data} . "/${name}";
 
 	if (not -e $pwfile) {
 		confess('Password file does not exist');
 	}
 
-	my %key = file_to_hash($pwfile);
+	my %key = $self->file_to_hash($pwfile);
 
 	$self->get_master_password();
 
-	$self->{'pass'}->salt($key{'salt'});
+	$self->{pass}->salt($key{salt});
 
-	$self->ui()->to_clipboard($self->{'pass'}->decrypt($key{'hash'}));
+	$self->ui()->to_clipboard($self->{pass}->decrypt($key{hash}));
 
-	if ($key{'extra'}) {
-		print $self->{'pass'}->decrypt($key{'extra'})
+	if ($key{extra}) {
+		print $self->{pass}->decrypt($key{extra})
 	}
 }
 
-=head2 ->cmd_info($name)
+=item $raps2->cmd_info(I<$name>)
 
 Prints unencrypted information about $name.
 
@@ -315,15 +314,48 @@ Prints unencrypted information about $name.
 
 sub cmd_info {
 	my ($self, $name) = @_;
-	my $pwfile = $self->{'xdg_data'} . "/${name}";
+	my $pwfile = $self->{xdg_data} . "/${name}";
 
 	if (not -e $pwfile) {
 		confess('Password file does not exist');
 	}
 
-	my %key = file_to_hash($pwfile);
+	my %key = $self->file_to_hash($pwfile);
 	$self->ui()->output(
-		['URL', $key{'url'}],
-		['Login', $key{'login'}],
+		['URL', $key{url}],
+		['Login', $key{login}],
 	);
 }
+
+=item $raps2->cmd_list()
+
+Lists all saved passwords and their logins and urls
+
+=cut
+
+sub cmd_list {
+	my ($self) = @_;
+
+	for my $file (read_dir($self->{xdg_data})) {
+		my %key = $self->file_to_hash($self->{xdg_data} . "/${file}");
+		$self->ui->list(
+			['Account', $file],
+			['Login', $key{login}],
+			['URL', $key{url}],
+		);
+	}
+}
+
+=back
+
+=head1 DEPENDENCIES
+
+L<App::Raps2::Password>, L<App::Raps2::UI>, L<File::BaseDir>, L<File::Slurp>.
+
+=head1 AUTHOR
+
+Copyright (C) 2011 by Daniel Friesel E<lt>derf@finalrewind.orgE<gt>
+
+=head1 LICENSE
+
+  0. You just DO WHAT THE FUCK YOU WANT TO.
